clear ;
%
%  CAR PACKING VIA FLATNESS AND FRENET FORMULAS
%
%    explicit computation and visualisation of the motions.
%
%    February 1993
%
% ............................................................
% :         Pierre ROUCHON  <rouchon@cas.ensmp.fr>           :
% : Centre Automatique et Systemes, Ecole des Mines de Paris :
% : 60, bd Saint Michel -- 75272 PARIS CEDEX 06, France      :
% :    Telephone: (1) 40 51 91 15 --- Fax: (1) 43 54 18 93   :
% :..........................................................:
%
%
% L:  car length (m) 
% T: basic time interval for one  smooth motion (s)
% a0, a1, p(3): intermediate variables for polynomial 
%           curves definition
%
global T L a0 a1 p  ;
%
T = 1 ; L = 1 ;
a0 =0 ; a1 = 0 ;
p= [0 0 0 ] ;
%
% initial configuration of the car
x1 =-3 ; y1 = 2 ; theta1 = pi/10 ; phi1 = 0 ;
% final configuration of the car 
x2 = 0 ; y2 = 0  ; theta2 =0; phi2 = 0 ;
% Constraints: y1 > y2 and -pi/2 < theta1,2, phi1,2 < pi/2
%
%  sampling of motion 1 --> 0 and 0 --> 2 
     nbpt = 40 ;
% conputation of  intermediate configuration 
x0 = 2;
y0 = .8 ;
%
%
% first polynomial curve
a0 = x0 ; b0 = y0 ;
a1 = x1 ; b1 = y1 ; 
M = [ 
  (a1-a0)^3     (a1-a0)^4     (a1-a0)^5 
3*(a1-a0)^2   4*(a1-a0)^3   5*(a1-a0)^4
6*(a1-a0)    12*(a1-a0)^2  20*(a1-a0)^3 
      ] ;
q = [ 
   b1-b0 
   tan(theta1) 
   tan(phi1)/(L*(cos(theta1)^3)) 
    ] ;
p = inv(M)*q ;
%
% computation the first motion, time: 1 -> 0
 state=[ x1 y1 theta1 phi1 ] ;
 for i=1:(nbpt+1)
    tau = (i-1)/nbpt ;
    phi=tau*tau*(3-2*tau) ;
    a = (1-phi)*a1 + phi*a0 ;
   f=  b0+ p(1).*(a-a0)^3 +    p(2).*(a-a0)^4 +    p(3).*(a-a0)^5 ;
   df  = 3*p(1).*(a-a0)^2 +  4*p(2).*(a-a0)^3 +  5*p(3).*(a-a0)^4 ;
   ddf = 6*p(1).*(a-a0)   + 12*p(2).*(a-a0)^2 + 20*p(3).*(a-a0)^3 ;
   k = ddf / ((1+df*df)^(3/2)) ;
   state = [ state; a   f  atan(df) atan(k*L)] ; 
 end  
%
%
% second polynomial curve
a0 = x0 ; b0 = y0 ;
a1 = x2 ; b1 = y2 ; 
M = [ 
  (a1-a0)^3     (a1-a0)^4     (a1-a0)^5 
3*(a1-a0)^2   4*(a1-a0)^3   5*(a1-a0)^4
6*(a1-a0)    12*(a1-a0)^2  20*(a1-a0)^3 
      ] ;
q = [ 
      b1-b0 
   tan(theta2) 
   tan(phi2)/(L*(cos(theta2)^3)) 
    ] ;
p = inv(M)*q ;
%
% computation of the second motion 0 --> 2
 for i=1:(nbpt+1)
   tau = (i-1)/nbpt ;
   phi=tau*tau*(3-2*tau) ;
   a = (1-phi)*a0 + phi*a1 ;
   f=  b0+ p(1).*(a-a0)^3 +    p(2).*(a-a0)^4 +    p(3).*(a-a0)^5 ;
   df  = 3*p(1).*(a-a0)^2 +  4*p(2).*(a-a0)^3 +  5*p(3).*(a-a0)^4 ;
   ddf = 6*p(1).*(a-a0)   + 12*p(2).*(a-a0)^2 + 20*p(3).*(a-a0)^3 ;
   k = ddf / ((1+df*df)^(3/2)) ;
   state = [ state; a   f  atan(df) atan(k*L)] ;
 end  
%
%
% Graphics 
%
% window size
xmin = min(state(:,1))-0.5*L ;
xmax = max(state(:,1))+1.5*L ;
ymin = min(state(:,2))-1.5*L ;
ymax = max(state(:,2))+1.5*L ;
dd = max([xmax-xmin ymax-ymin]) ;
clg ;
axis('square') ;
axis([xmin xmin+dd ymin ymin+dd]) ;
plot([0 0],[0 0]) ;
hold on ;
clg ;
%
% starting configuration
plotcar(x1,y1,theta1,phi1,L,'-') ;
plot(x1,y1,'o') ;
pause;
%
% end configuration
plotcar(x2,y2,theta2,phi2,L,'-') ;
plot(x2,y2,'o') ;
pause ; 
%
% intermediate configuration (inversion of velocity)
plotcar(x0,y0,0,0,L,'-') ;
plot(x0,y0,'o') ;
pause ;
%
% trajectory of the linearizing output
plot(state(:,1),state(:,2)) ;
pause 
%
% movies 
[n m] = size(state) ;
for i=1:n
  plotcar( state(i,1),state(i,2), ....
            state(i,3),state(i,4),L,'-') ;
  plot(state(i,1),state(i,2),'o') ;
end ;



